/*
   Example to print first N prime numbers,
 */

#include <iostream>
#include <iomanip>
#include <vector>

using namespace std ;

// define N as a preprocess variable, will be substitued 
// by the preprocessing phase
#define N 1000

template <typename IntType> 
bool // boolean type, store true or false value only
// check if candidate is a prime number respect to the vector primes
// return true if primes[i] do not divide candidate for each i
checkPrime( vector<IntType> const & primes, IntType const & candidate ) {
  typename vector<IntType>::const_iterator i ;
  for ( i = primes . begin() ; i != primes.end() ; ++i ) {
    if ( ( candidate % (*i) ) == 0 ) // if remainder is 0 it is not a prime!
      return false ;
  }
  return true ;
}

template <typename IntType> 
void // void type, nothing to return!
addPrime( vector<IntType> & primes ) {
  IntType candidate = primes . back() + 1 ;
  // ! is the negation operator ! true = false.
  while ( ! checkPrime( primes, candidate) ) ++candidate ;
  primes . push_back(candidate) ;
}

template <typename IntType> 
void
printNumbers( ostream & s,
              vector<IntType> const & primes ) {
  int kk = 0 ;
  typename vector<IntType>::const_iterator i ;
  for ( i = primes . begin() ; i != primes.end() ; ++i ) {
    // print the number to the file fd (can be stdout)
    // no new line or others amenity
    s << setw(5) << *i ;
    ++kk ; // increment the counter
    
    // decide if to use comma of newline
    if ( (kk%10) == 0 ) s << "\n" ;
    else                s << ", " ;
  }
  s << "\n" ;
}

int
main ()
{
  vector<long int> primes ;
  primes . push_back(2) ;
  primes . push_back(3) ;
  primes . push_back(5) ;
  primes . push_back(7) ;
  
  do {
    addPrime( primes ) ; 
  } while ( primes . size() < N ) ; // chech if it must cycle again
  
  printNumbers( cout, primes ) ; // print the result to standard output
  
  return 0 ;
}

